package de.fhdw.gaming.ipspiel23.ht.domain.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.IntStream;

import de.fhdw.gaming.core.domain.DefaultObserverFactoryProvider;
import de.fhdw.gaming.core.domain.GameBuilder;
import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.core.domain.ObserverFactoryProvider;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTGame;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTGameBuilder;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTPlayer;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTPlayerBuilder;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTState;
import de.fhdw.gaming.ipspiel23.ht.moves.HTBaseMove;
import de.fhdw.gaming.ipspiel23.ht.strategy.IHTStrategy;

/**
 * Represents a builder for {@link IHTGame}s.
 */
final class HTGameBuilder implements IHTGameBuilder {
    
    /**
     * The {@link ObserverFactoryProvider}.
     */
    private final ObserverFactoryProvider observerFactoryProvider;

    /**
     * The players of the game.
     */
    private final List<IHTPlayer> players = new ArrayList<>(2);

    /**
     * The strategies of the players.
     */
    private final List<IHTStrategy> playerStrategies = new ArrayList<>(2);
    
    /**
     * The maximum computation time per move in seconds.
     */
    private int maxComputationTimePerMove;
    
    /**
     * Creates a new instance of {@link HTGameBuilder}.
     */
    public HTGameBuilder() {
        observerFactoryProvider = new DefaultObserverFactoryProvider();
        maxComputationTimePerMove = GameBuilder.DEFAULT_MAX_COMPUTATION_TIME_PER_MOVE;
    }

    @Override
    public HTGameBuilder changeMaximumComputationTimePerMove(final int newMaxComputationTimePerMove) {
        this.maxComputationTimePerMove = newMaxComputationTimePerMove;
        return this;
    }

    @Override
    public IHTGame build(final int id) throws GameException, InterruptedException {
        if (players.size() != 2 || playerStrategies.size() != 2) {
            throw new GameException("Exactly two players are required for 'Heads or Tails'.");
        }

        final IHTState initial = new HTState(this.players.get(0), this.players.get(1));
        final Map<String, IHTStrategy> strategies = new LinkedHashMap<>(2);
        IntStream.range(0, 2)
            .forEachOrdered(i -> strategies.put(this.players.get(i).getName(), this.playerStrategies.get(i)));
        return new HTGame(id, initial, strategies, this.maxComputationTimePerMove, 
            HTBaseMove.class::isInstance, this.observerFactoryProvider);
    }

    @Override
    public IHTPlayerBuilder createPlayerBuilder() {
        return new HTPlayerBuilder();
    }

    @Override
    public IHTGameBuilder addPlayer(final IHTPlayer player, final IHTStrategy strategy) throws GameException {
        if (this.players.size() >= 2) {
            throw new GameException("Cannot handle more than two players.");
        }
        this.players.add(player);
        this.playerStrategies.add(strategy);
        return this;
    }
}