package de.fhdw.gaming.ipspiel23.ht.strategy.impl.mixed;

import java.util.Optional;
import java.util.Random;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTPlayer;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTState;
import de.fhdw.gaming.ipspiel23.ht.moves.IHTMove;
import de.fhdw.gaming.ipspiel23.ht.moves.factory.IHTMoveFactory;
import de.fhdw.gaming.ipspiel23.ht.strategy.impl.HTStrategy;

/**
 * 
 * implements {@link HTStrategy} by choosing one of the three possible answers.
 *
 */
public class HTMixedStrategy extends HTStrategy {

    /**
     * Random to determine the Answer to Choose.
     */
    private static final Random RND = new Random();

    /**
     * Constructor.
     * 
     * @param moveFactory moveFactory.
     */
    protected HTMixedStrategy(final IHTMoveFactory moveFactory) {
        super(moveFactory);
    }

    @Override
    public Optional<IHTMove> computeNextMove(final int gameId, final IHTPlayer player, final IHTState state)
            throws GameException, InterruptedException {
        switch (RND.nextInt(3)) {
        case 0: {
            return Optional.of(getMoveFactory().createHeadsMove());
        }
        case 1: {
            return Optional.of(getMoveFactory().createTailsMove());
        }
        case 2: {
            return Optional.of(getMoveFactory().createEdgeMove());
        }
        default: {
            throw new IllegalArgumentException("Value has to be in Range: 0-2.");
        }
        }
    }
}
